@@ -2,7 +2,7 @@
 #
 # This file is part of Glances.
 #
-# Copyright (C) 2019 Nicolargo <nicolas@nicolargo.com>
+# Copyright (C) 2021 Nicolargo <nicolas@nicolargo.com>
 #
 # Glances is free software; you can redistribute it and/or modify
 # it under the terms of the GNU Lesser General Public License as published by
@@ -19,6 +19,7 @@
 
 """CPU percent stats shared between CPU and Quicklook plugins."""
 
+from glances.logger import logger
 from glances.timer import Timer
 
 import psutil
@@ -28,15 +29,24 @@ class CpuPercent(object):
 
     """Get and store the CPU percent."""
 
-    def __init__(self, cached_time=1):
+    def __init__(self, cached_timer_cpu=3):
+        self.cpu_info = {}
         self.cpu_percent = 0
         self.percpu_percent = []
 
-        # cached_time is the minimum time interval between stats updates
+        # Get CPU name
+        self.__get_cpu_name()
+
+        # cached_timer_cpu is the minimum time interval between stats updates
         # since last update is passed (will retrieve old cached info instead)
+        self.cached_timer_cpu = cached_timer_cpu
         self.timer_cpu = Timer(0)
         self.timer_percpu = Timer(0)
-        self.cached_time = cached_time
+
+        # psutil.cpu_freq() consumes lots of CPU
+        # So refresh the stats every refresh*2 (6 seconds)
+        self.cached_timer_cpu_info = cached_timer_cpu * 2
+        self.timer_cpu_info = Timer(0)
 
     def get_key(self):
         """Return the key of the per CPU list."""
@@ -50,21 +60,49 @@ class CpuPercent(object):
         else:
             return self.__get_cpu()
 
+    def get_info(self):
+        """Get additional informations about the CPU"""
+        # Never update more than 1 time per cached_timer_cpu_info
+        if self.timer_cpu_info.finished() and hasattr(psutil, 'cpu_freq'):
+            # Get the CPU freq current/max
+            cpu_freq = psutil.cpu_freq()
+            if hasattr(cpu_freq, 'current'):
+                self.cpu_info['cpu_hz_current'] = cpu_freq.current
+            else:
+                self.cpu_info['cpu_hz_current'] = None
+            if hasattr(cpu_freq, 'max'):
+                self.cpu_info['cpu_hz'] = cpu_freq.max
+            else:
+                self.cpu_info['cpu_hz'] = None
+            # Reset timer for cache
+            self.timer_cpu_info.reset(duration=self.cached_timer_cpu_info)
+        return self.cpu_info
+
+    def __get_cpu_name(self):
+        # Get the CPU name once from the /proc/cpuinfo file
+        # @TODO: Multisystem...
+        try:
+            self.cpu_info['cpu_name'] = open('/proc/cpuinfo', 'r').readlines()[4].split(':')[1][1:-2]
+        except:
+            self.cpu_info['cpu_name'] = 'CPU'
+        return self.cpu_info['cpu_name']
+
     def __get_cpu(self):
         """Update and/or return the CPU using the psutil library."""
-        # Never update more than 1 time per cached_time
+        # Never update more than 1 time per cached_timer_cpu
         if self.timer_cpu.finished():
             self.cpu_percent = psutil.cpu_percent(interval=0.0)
             # Reset timer for cache
-            self.timer_cpu = Timer(self.cached_time)
+            self.timer_cpu.reset(duration=self.cached_timer_cpu)
         return self.cpu_percent
 
     def __get_percpu(self):
         """Update and/or return the per CPU list using the psutil library."""
-        # Never update more than 1 time per cached_time
+        # Never update more than 1 time per cached_timer_cpu
         if self.timer_percpu.finished():
             self.percpu_percent = []
-            for cpu_number, cputimes in enumerate(psutil.cpu_times_percent(interval=0.0, percpu=True)):
+            for cpu_number, cputimes in enumerate(psutil.cpu_times_percent(interval=0.0,
+                                                                           percpu=True)):
                 cpu = {'key': self.get_key(),
                        'cpu_number': cpu_number,
                        'total': round(100 - cputimes.idle, 1),
@@ -89,7 +127,7 @@ class CpuPercent(object):
                 # Append new CPU to the list
                 self.percpu_percent.append(cpu)
                 # Reset timer for cache
-                self.timer_percpu = Timer(self.cached_time)
+                self.timer_percpu.reset(duration=self.cached_timer_cpu)
         return self.percpu_percent
 
 
