@@ -2,7 +2,7 @@
 #
 # This file is part of Glances.
 #
-# Copyright (C) 2019 Nicolargo <nicolas@nicolargo.com>
+# Copyright (C) 2021 Nicolargo <nicolas@nicolargo.com>
 #
 # Glances is free software; you can redistribute it and/or modify
 # it under the terms of the GNU Lesser General Public License as published by
@@ -19,6 +19,7 @@
 
 """CPU plugin."""
 
+from glances.logger import logger
 from glances.timer import getTimeSinceLastUpdate
 from glances.compat import iterkeys
 from glances.cpu_percent import cpu_percent
@@ -59,9 +60,11 @@ class Plugin(GlancesPlugin):
     percentage.
     """
 
-    def __init__(self, args=None):
+    def __init__(self, args=None, config=None):
         """Init the CPU plugin."""
-        super(Plugin, self).__init__(args=args, items_history_list=items_history_list)
+        super(Plugin, self).__init__(args=args,
+                                     config=config,
+                                     items_history_list=items_history_list)
 
         # We want to display the stat in the curse interface
         self.display_curse = True
@@ -76,7 +79,6 @@ class Plugin(GlancesPlugin):
     @GlancesPlugin._log_result_decorator
     def update(self):
         """Update CPU stats using the input method."""
-
         # Grab stats into self.stats
         if self.input_method == 'local':
             stats = self.update_local()
@@ -102,39 +104,41 @@ class Plugin(GlancesPlugin):
         stats = self.get_init_value()
 
         stats['total'] = cpu_percent.get()
+        # Grab: 'user', 'system', 'idle', 'nice', 'iowait',
+        #       'irq', 'softirq', 'steal', 'guest', 'guest_nice'
         cpu_times_percent = psutil.cpu_times_percent(interval=0.0)
-        for stat in ['user', 'system', 'idle', 'nice', 'iowait',
-                     'irq', 'softirq', 'steal', 'guest', 'guest_nice']:
-            if hasattr(cpu_times_percent, stat):
-                stats[stat] = getattr(cpu_times_percent, stat)
+        for stat in cpu_times_percent._fields:
+            stats[stat] = getattr(cpu_times_percent, stat)
 
         # Additional CPU stats (number of events not as a %; psutil>=4.1.0)
-        # ctx_switches: number of context switches (voluntary + involuntary) per second
-        # interrupts: number of interrupts per second
-        # soft_interrupts: number of software interrupts per second. Always set to 0 on Windows and SunOS.
-        # syscalls: number of system calls since boot. Always set to 0 on Linux.
+        # - ctx_switches: number of context switches (voluntary + involuntary) since boot.
+        # - interrupts: number of interrupts since boot.
+        # - soft_interrupts: number of software interrupts since boot. Always set to 0 on Windows and SunOS.
+        # - syscalls: number of system calls since boot. Always set to 0 on Linux.
         cpu_stats = psutil.cpu_stats()
+
         # By storing time data we enable Rx/s and Tx/s calculations in the
         # XML/RPC API, which would otherwise be overly difficult work
         # for users of the API
-        time_since_update = getTimeSinceLastUpdate('cpu')
+        stats['time_since_update'] = getTimeSinceLastUpdate('cpu')
+
+        # Core number is needed to compute the CTX switch limit
+        stats['cpucore'] = self.nb_log_core
 
         # Previous CPU stats are stored in the cpu_stats_old variable
         if not hasattr(self, 'cpu_stats_old'):
-            # First call, we init the cpu_stats_old var
-            self.cpu_stats_old = cpu_stats
+            # Init the stats (needed to have the key name for export)
+            for stat in cpu_stats._fields:
+                # @TODO: better to set it to None but should refactor views and UI...
+                stats[stat] = 0
         else:
+            # Others calls...
             for stat in cpu_stats._fields:
                 if getattr(cpu_stats, stat) is not None:
                     stats[stat] = getattr(cpu_stats, stat) - getattr(self.cpu_stats_old, stat)
 
-            stats['time_since_update'] = time_since_update
-
-            # Core number is needed to compute the CTX switch limit
-            stats['cpucore'] = self.nb_log_core
-
-            # Save stats to compute next step
-            self.cpu_stats_old = cpu_stats
+        # Save stats to compute next step
+        self.cpu_stats_old = cpu_stats
 
         return stats
 
@@ -194,11 +198,11 @@ class Plugin(GlancesPlugin):
 
         # Add specifics informations
         # Alert and log
-        for key in ['user', 'system', 'iowait']:
+        for key in ['user', 'system', 'iowait', 'total']:
             if key in self.stats:
                 self.views[key]['decoration'] = self.get_alert_log(self.stats[key], header=key)
         # Alert only
-        for key in ['steal', 'total']:
+        for key in ['steal']:
             if key in self.stats:
                 self.views[key]['decoration'] = self.get_alert(self.stats[key], header=key)
         # Alert only but depend on Core number
@@ -206,7 +210,7 @@ class Plugin(GlancesPlugin):
             if key in self.stats:
                 self.views[key]['decoration'] = self.get_alert(self.stats[key], maximum=100 * self.stats['cpucore'], header=key)
         # Optional
-        for key in ['nice', 'irq', 'iowait', 'steal', 'ctx_switches', 'interrupts', 'soft_interrupts', 'syscalls']:
+        for key in ['nice', 'irq', 'idle', 'steal', 'ctx_switches', 'interrupts', 'soft_interrupts', 'syscalls']:
             if key in self.stats:
                 self.views[key]['optional'] = True
 
@@ -237,17 +241,14 @@ class Plugin(GlancesPlugin):
         ret.append(self.curse_add_line(msg))
         # Total CPU usage
         msg = '{:5.1f}%'.format(self.stats['total'])
-        if idle_tag:
-            ret.append(self.curse_add_line(
-                msg, self.get_views(key='total', option='decoration')))
-        else:
+        ret.append(self.curse_add_line(
+            msg, self.get_views(key='total', option='decoration')))
+        # Idle CPU
+        if 'idle' in self.stats and not idle_tag:
+            msg = '  {:8}'.format('idle:')
+            ret.append(self.curse_add_line(msg))
+            msg = '{:5.1f}%'.format(self.stats['idle'])
             ret.append(self.curse_add_line(msg))
-        # Nice CPU
-        if 'nice' in self.stats:
-            msg = '  {:8}'.format('nice:')
-            ret.append(self.curse_add_line(msg, optional=self.get_views(key='nice', option='optional')))
-            msg = '{:5.1f}%'.format(self.stats['nice'])
-            ret.append(self.curse_add_line(msg, optional=self.get_views(key='nice', option='optional')))
         # ctx_switches
         if 'ctx_switches' in self.stats:
             msg = '  {:8}'.format('ctx_sw:')
@@ -299,14 +300,14 @@ class Plugin(GlancesPlugin):
             ret.append(self.curse_add_line(msg))
             msg = '{:>6}'.format(self.stats['nb_log_core'])
             ret.append(self.curse_add_line(msg))
-        # IOWait CPU
-        if 'iowait' in self.stats:
-            msg = '  {:8}'.format('iowait:')
-            ret.append(self.curse_add_line(msg, optional=self.get_views(key='iowait', option='optional')))
-            msg = '{:5.1f}%'.format(self.stats['iowait'])
+        # Nice CPU
+        if 'nice' in self.stats:
+            msg = '  {:8}'.format('nice:')
             ret.append(self.curse_add_line(
-                msg, self.get_views(key='iowait', option='decoration'),
-                optional=self.get_views(key='iowait', option='optional')))
+                msg, optional=self.get_views(key='nice', option='optional')))
+            msg = '{:5.1f}%'.format(self.stats['nice'])
+            ret.append(self.curse_add_line(
+                msg, optional=self.get_views(key='nice', option='optional')))
         # soft_interrupts
         if 'soft_interrupts' in self.stats:
             msg = '  {:8}'.format('sw_int:')
@@ -316,12 +317,15 @@ class Plugin(GlancesPlugin):
 
         # New line
         ret.append(self.curse_new_line())
-        # Idle CPU
-        if 'idle' in self.stats and not idle_tag:
-            msg = '{:8}'.format('idle:')
-            ret.append(self.curse_add_line(msg))
-            msg = '{:5.1f}%'.format(self.stats['idle'])
-            ret.append(self.curse_add_line(msg))
+        # IOWait CPU
+        if 'iowait' in self.stats:
+            msg = '{:8}'.format('iowait:')
+            ret.append(self.curse_add_line(
+                msg, optional=self.get_views(key='iowait', option='optional')))
+            msg = '{:5.1f}%'.format(self.stats['iowait'])
+            ret.append(self.curse_add_line(
+                msg, self.get_views(key='iowait', option='decoration'),
+                optional=self.get_views(key='iowait', option='optional')))
         # Steal CPU usage
         if 'steal' in self.stats:
             msg = '  {:8}'.format('steal:')
