@@ -22,6 +22,7 @@
 import os
 import threading
 import time
+from copy import deepcopy
 
 from glances.logger import logger
 from glances.compat import iterkeys, itervalues, nativestr
@@ -66,6 +67,10 @@ items_history_list = [{'name': 'cpu_percent',
                        'y_unit': '%'}]
 
 
+# List of key to remove before export
+export_exclude_list = ['cpu', 'io', 'memory', 'network']
+
+
 class Plugin(GlancesPlugin):
     """Glances Docker plugin.
 
@@ -81,6 +86,9 @@ class Plugin(GlancesPlugin):
         # The plgin can be disable using: args.disable_docker
         self.args = args
 
+        # Default config keys
+        self.config = config
+
         # We want to display the stat in the curse interface
         self.display_curse = True
 
@@ -92,6 +100,10 @@ class Plugin(GlancesPlugin):
         # value: instance of ThreadDockerGrabber
         self.thread_list = {}
 
+        # Force a first update because we need two update to have the first stat
+        self.update()
+        self.refresh_timer.set(0)
+
     def exit(self):
         """Overwrite the exit method to close threads."""
         for t in itervalues(self.thread_list):
@@ -109,16 +121,25 @@ class Plugin(GlancesPlugin):
         - Only exports containers
         - The key is the first container name
         """
-        ret = []
         try:
-            ret = self.stats['containers']
+            ret = deepcopy(self.stats['containers'])
         except KeyError as e:
             logger.debug("docker plugin - Docker export error {}".format(e))
+            ret = []
+
+        # Remove fields uses to compute rate
+        for container in ret:
+            for i in export_exclude_list:
+                container.pop(i)
+
         return ret
 
     def connect(self):
         """Connect to the Docker server."""
         try:
+            # If the following line replace the next one, the issue #1878
+            # is reproduced (Docker containers information missing with Docker 20.10.x)
+            # So, for the moment disable the timeout option
             ret = docker.from_env()
         except Exception as e:
             logger.error("docker plugin - Can not connect to Docker ({})".format(e))
@@ -146,8 +167,8 @@ class Plugin(GlancesPlugin):
         # Init new stats
         stats = self.get_init_value()
 
-        # The Docker-py lib is mandatory
-        if import_error_tag:
+        # The Docker-py lib is mandatory and connection should be ok
+        if import_error_tag or self.docker_client is None:
             return self.stats
 
         if self.input_method == 'local':
@@ -207,6 +228,14 @@ class Plugin(GlancesPlugin):
             # Get stats for all containers
             stats['containers'] = []
             for container in containers:
+                # Only show specific containers
+                if not self.is_show(nativestr(container.name)):
+                    continue
+
+                # Do not take hiden container into account
+                if self.is_hide(nativestr(container.name)):
+                    continue
+
                 # Init the stats for the current container
                 container_stats = {}
                 # The key is the container name and not the Id
@@ -223,14 +252,22 @@ class Plugin(GlancesPlugin):
                 container_stats['Status'] = container.attrs['State']['Status']
                 container_stats['Command'] = container.attrs['Config']['Entrypoint']
                 # Standards stats
+                # See https://docs.docker.com/engine/api/v1.41/#operation/ContainerStats
+                # Be aware that the API can change... (example see issue #1857)
                 if container_stats['Status'] in ('running', 'paused'):
+                    # CPU
                     container_stats['cpu'] = self.get_docker_cpu(container.id, self.thread_list[container.id].stats)
                     container_stats['cpu_percent'] = container_stats['cpu'].get('total', None)
+                    # MEM
                     container_stats['memory'] = self.get_docker_memory(container.id, self.thread_list[container.id].stats)
                     container_stats['memory_usage'] = container_stats['memory'].get('usage', None)
+                    if container_stats['memory'].get('cache', None) is not None:
+                        container_stats['memory_usage'] -= container_stats['memory']['cache']
+                    # IO
                     container_stats['io'] = self.get_docker_io(container.id, self.thread_list[container.id].stats)
                     container_stats['io_r'] = container_stats['io'].get('ior', None)
                     container_stats['io_w'] = container_stats['io'].get('iow', None)
+                    # NET
                     container_stats['network'] = self.get_docker_network(container.id, self.thread_list[container.id].stats)
                     container_stats['network_rx'] = container_stats['network'].get('rx', None)
                     container_stats['network_tx'] = container_stats['network'].get('tx', None)
@@ -265,7 +302,6 @@ class Plugin(GlancesPlugin):
                all_stats is the output of the stats method of the Docker API
         Output: a dict {'total': 1.49}
         """
-        cpu_new = {}
         ret = {'total': 0.0}
 
         # Read the stats
@@ -273,38 +309,41 @@ class Plugin(GlancesPlugin):
         # containing the CPU usage accumulated by the processes of the container.
         # Those times are expressed in ticks of 1/USER_HZ of a second.
         # On x86 systems, USER_HZ is 100.
+        cpu_new = {}
+        precpu_new = {}
         try:
-            cpu_new['total'] = all_stats['cpu_stats']['cpu_usage']['total_usage']
-            cpu_new['system'] = all_stats['cpu_stats']['system_cpu_usage']
-            cpu_new['nb_core'] = len(all_stats['cpu_stats']['cpu_usage']['percpu_usage'] or [])
+            cpu_new['total'] = all_stats['cpu_stats']['cpu_usage'].get(
+                'total_usage', None)
+            precpu_new['total'] = all_stats['precpu_stats']['cpu_usage'].get(
+                'total_usage', None)
+            cpu_new['system'] = all_stats['cpu_stats'].get(
+                'system_cpu_usage', None)
+            precpu_new['system'] = all_stats['precpu_stats'].get(
+                'system_cpu_usage', None)
+            # Issue #1857
+            # If either precpu_stats.online_cpus or cpu_stats.online_cpus is nil
+            # then for compatibility with older daemons the length of
+            # the corresponding cpu_usage.percpu_usage array should be used.
+            if 'online_cpus' in all_stats['cpu_stats'] and \
+               all_stats['cpu_stats']['online_cpus'] is not None:
+                cpu_new['nb_core'] = all_stats['cpu_stats']['online_cpus']
+            else:
+                cpu_new['nb_core'] = len(all_stats['cpu_stats']['cpu_usage']['percpu_usage'] or [])
         except KeyError as e:
-            # all_stats do not have CPU information
-            logger.debug("docker plugin - Cannot grab CPU usage for container {} ({})".format(container_id, e))
+            logger.debug(
+                "docker plugin - Cannot grab CPU usage for container {} ({})".format(container_id, e))
             logger.debug(all_stats)
         else:
-            # Previous CPU stats stored in the cpu_old variable
-            if not hasattr(self, 'cpu_old'):
-                # First call, we init the cpu_old variable
-                self.cpu_old = {}
-                try:
-                    self.cpu_old[container_id] = cpu_new
-                except (IOError, UnboundLocalError):
-                    pass
-
-            if container_id not in self.cpu_old:
-                try:
-                    self.cpu_old[container_id] = cpu_new
-                except (IOError, UnboundLocalError):
-                    pass
-            else:
-                #
-                cpu_delta = float(cpu_new['total'] - self.cpu_old[container_id]['total'])
-                system_delta = float(cpu_new['system'] - self.cpu_old[container_id]['system'])
-                if cpu_delta > 0.0 and system_delta > 0.0:
-                    ret['total'] = (cpu_delta / system_delta) * float(cpu_new['nb_core']) * 100
-
-                # Save stats to compute next stats
-                self.cpu_old[container_id] = cpu_new
+            try:
+                cpu_delta = cpu_new['total'] - precpu_new['total']
+                system_cpu_delta = cpu_new['system'] - precpu_new['system']
+                # CPU usage % = (cpu_delta / system_cpu_delta) * number_cpus * 100.0
+                ret['total'] = (cpu_delta / system_cpu_delta) * \
+                    cpu_new['nb_core'] * 100.0
+            except TypeError as e:
+                logger.debug(
+                    "docker plugin - Cannot compute CPU usage for container {} ({})".format(container_id, e))
+                logger.debug(all_stats)
 
         # Return the stats
         return ret
@@ -319,11 +358,21 @@ class Plugin(GlancesPlugin):
         ret = {}
         # Read the stats
         try:
-            ret['rss'] = all_stats['memory_stats']['stats']['rss']
-            ret['cache'] = all_stats['memory_stats']['stats']['cache']
+            # Issue #1857
+            # Some stats are not always available in ['memory_stats']['stats']
+            if 'rss' in all_stats['memory_stats']['stats']:
+                ret['rss'] = all_stats['memory_stats']['stats']['rss']
+            elif 'total_rss' in all_stats['memory_stats']['stats']:
+                ret['rss'] = all_stats['memory_stats']['stats']['total_rss']
+            else:
+                ret['rss'] = None
+            ret['cache'] = all_stats['memory_stats']['stats'].get(
+                'cache', None)
+            ret['max_usage'] = all_stats['memory_stats'].get(
+                'max_usage', None)
+            # Mandatory fields
             ret['usage'] = all_stats['memory_stats']['usage']
             ret['limit'] = all_stats['memory_stats']['limit']
-            ret['max_usage'] = all_stats['memory_stats']['max_usage']
         except (KeyError, TypeError) as e:
             # all_stats do not have MEM information
             logger.debug("docker plugin - Cannot grab MEM usage for container {} ({})".format(container_id, e))
@@ -524,8 +573,14 @@ class Plugin(GlancesPlugin):
         ret.append(self.curse_new_line())
         # Header
         ret.append(self.curse_new_line())
-        # Get the maximum containers name (cutted to 20 char max)
-        name_max_width = min(20, len(max(self.stats['containers'], key=lambda x: len(x['name']))['name']))
+        # Get the maximum containers name
+        # Max size is configurable. See feature request #1723.
+        name_max_width = min(self.config.get_int_value('docker',
+                                                       'max_name_size',
+                                                       default=20)
+                             if self.config is not None else 20,
+                             len(max(self.stats['containers'],
+                                 key=lambda x: len(x['name']))['name']))
         msg = ' {:{width}}'.format('Name', width=name_max_width)
         ret.append(self.curse_add_line(msg))
         msg = '{:>10}'.format('Status')
