@@ -25,6 +25,7 @@ import psutil
 from glances.compat import iteritems
 from glances.plugins.glances_core import Plugin as CorePlugin
 from glances.plugins.glances_plugin import GlancesPlugin
+from glances.logger import logger
 
 # SNMP OID
 # 1 minute Load: .1.3.6.1.4.1.2021.10.1.3.1
@@ -50,9 +51,10 @@ class Plugin(GlancesPlugin):
     stats is a dict
     """
 
-    def __init__(self, args=None):
+    def __init__(self, args=None, config=None):
         """Init the plugin."""
         super(Plugin, self).__init__(args=args,
+                                     config=config,
                                      items_history_list=items_history_list)
 
         # We want to display the stat in the curse interface
@@ -61,7 +63,8 @@ class Plugin(GlancesPlugin):
         # Call CorePlugin in order to display the core number
         try:
             self.nb_log_core = CorePlugin(args=self.args).update()["log"]
-        except Exception:
+        except Exception as e:
+            logger.warning('Error: Can not retrieve the CPU core number (set it to 1) ({})'.format(e))
             self.nb_log_core = 1
 
     def _getloadavg(self):
@@ -123,9 +126,11 @@ class Plugin(GlancesPlugin):
         # Add specifics informations
         try:
             # Alert and log
-            self.views['min15']['decoration'] = self.get_alert_log(self.stats['min15'], maximum=100 * self.stats['cpucore'])
+            self.views['min15']['decoration'] = self.get_alert_log(self.stats['min15'],
+                                                                   maximum=100 * self.stats['cpucore'])
             # Alert only
-            self.views['min5']['decoration'] = self.get_alert(self.stats['min5'], maximum=100 * self.stats['cpucore'])
+            self.views['min5']['decoration'] = self.get_alert(self.stats['min5'],
+                                                              maximum=100 * self.stats['cpucore'])
         except KeyError:
             # try/except mandatory for Windows compatibility (no load stats)
             pass
@@ -141,34 +146,29 @@ class Plugin(GlancesPlugin):
 
         # Build the string message
         # Header
-        msg = '{:8}'.format('LOAD')
+        msg = '{:6}'.format('LOAD%' if (args.disable_irix and self.nb_log_core != 0) else 'LOAD')
         ret.append(self.curse_add_line(msg, "TITLE"))
         # Core number
         if 'cpucore' in self.stats and self.stats['cpucore'] > 0:
-            msg = '{}-core'.format(int(self.stats['cpucore']))
+            msg = '{:3}-core'.format(int(self.stats['cpucore']))
             ret.append(self.curse_add_line(msg))
-        # New line
-        ret.append(self.curse_new_line())
-        # 1min load
-        msg = '{:8}'.format('1 min:')
-        ret.append(self.curse_add_line(msg))
-        msg = '{:>6.2f}'.format(self.stats['min1'])
-        ret.append(self.curse_add_line(msg))
-        # New line
-        ret.append(self.curse_new_line())
-        # 5min load
-        msg = '{:8}'.format('5 min:')
-        ret.append(self.curse_add_line(msg))
-        msg = '{:>6.2f}'.format(self.stats['min5'])
-        ret.append(self.curse_add_line(
-            msg, self.get_views(key='min5', option='decoration')))
-        # New line
-        ret.append(self.curse_new_line())
-        # 15min load
-        msg = '{:8}'.format('15 min:')
-        ret.append(self.curse_add_line(msg))
-        msg = '{:>6.2f}'.format(self.stats['min15'])
-        ret.append(self.curse_add_line(
-            msg, self.get_views(key='min15', option='decoration')))
+        # Loop over 1min, 5min and 15min load
+        for load_time in ['1', '5', '15']:
+            ret.append(self.curse_new_line())
+            msg = '{:8}'.format('{} min:'.format(load_time))
+            ret.append(self.curse_add_line(msg))
+            if args.disable_irix and self.nb_log_core != 0:
+                # Enable Irix mode for load (see issue #1554)
+                load_stat = self.stats['min{}'.format(load_time)] / self.nb_log_core * 100
+            else:
+                load_stat = self.stats['min{}'.format(load_time)]
+            msg = '{:>6.2f}'.format(load_stat)
+            if load_time == '1':
+                ret.append(self.curse_add_line(msg))
+            else:
+                # Alert is only for 5 and 15 min
+                ret.append(self.curse_add_line(
+                    msg, self.get_views(key='min{}'.format(load_time),
+                                        option='decoration')))
 
         return ret
