@@ -19,6 +19,7 @@
 
 """Virtual memory plugin."""
 
+from glances.logger import logger
 from glances.compat import iterkeys
 from glances.plugins.glances_plugin import GlancesPlugin
 
@@ -59,9 +60,10 @@ class Plugin(GlancesPlugin):
     stats is a dict
     """
 
-    def __init__(self, args=None):
+    def __init__(self, args=None, config=None):
         """Init the plugin."""
         super(Plugin, self).__init__(args=args,
+                                     config=config,
                                      items_history_list=items_history_list)
 
         # We want to display the stat in the curse interface
@@ -81,10 +83,15 @@ class Plugin(GlancesPlugin):
 
             # Get all the memory stats (copy/paste of the psutil documentation)
             # total: total physical memory available.
-            # available: the actual amount of available memory that can be given instantly to processes that request more memory in bytes; this is calculated by summing different memory values depending on the platform (e.g. free + buffers + cached on Linux) and it is supposed to be used to monitor actual memory usage in a cross platform fashion.
+            # available: the actual amount of available memory that can be given instantly
+            # to processes that request more memory in bytes; this is calculated by summing
+            # different memory values depending on the platform (e.g. free + buffers + cached on Linux)
+            # and it is supposed to be used to monitor actual memory usage in a cross platform fashion.
             # percent: the percentage usage calculated as (total - available) / total * 100.
-            # used: memory used, calculated differently depending on the platform and designed for informational purposes only.
-            # free: memory not being used at all (zeroed) that is readily available; note that this doesn’t reflect the actual memory available (use ‘available’ instead).
+            # used: memory used, calculated differently depending on the platform and designed for informational
+            # purposes only.
+            # free: memory not being used at all (zeroed) that is readily available; note that this doesn’t
+            # reflect the actual memory available (use ‘available’ instead).
             # Platform-specific fields:
             # active: (UNIX): memory currently in use or very recently used, and so it is in RAM.
             # inactive: (UNIX): memory that is marked as not used.
@@ -160,7 +167,7 @@ class Plugin(GlancesPlugin):
 
         # Add specifics informations
         # Alert and log
-        self.views['used']['decoration'] = self.get_alert_log(self.stats['used'], maximum=self.stats['total'])
+        self.views['percent']['decoration'] = self.get_alert_log(self.stats['used'], maximum=self.stats['total'])
         # Optional
         for key in ['active', 'inactive', 'buffers', 'cached']:
             if key in self.stats:
@@ -183,7 +190,8 @@ class Plugin(GlancesPlugin):
         ret.append(self.curse_add_line(msg))
         # Percent memory usage
         msg = '{:>7.1%}'.format(self.stats['percent'] / 100)
-        ret.append(self.curse_add_line(msg))
+        ret.append(self.curse_add_line(
+            msg, self.get_views(key='percent', option='decoration')))
         # Active memory usage
         if 'active' in self.stats:
             msg = '  {:9}'.format('active:')
