@@ -21,11 +21,10 @@
 from __future__ import unicode_literals
 
 import base64
-import operator
 
 from glances.timer import getTimeSinceLastUpdate
 from glances.plugins.glances_plugin import GlancesPlugin
-from glances.compat import n, u, b, nativestr
+from glances.compat import n
 from glances.logger import logger
 
 import psutil
@@ -62,11 +61,23 @@ class Plugin(GlancesPlugin):
         # We want to display the stat in the curse interface
         self.display_curse = True
 
+        # Hide stats if it has never been != 0
+        if config is not None:
+            self.hide_zero = config.get_bool_value(
+                self.plugin_name, 'hide_zero', default=False)
+        else:
+            self.hide_zero = False
+        self.hide_zero_fields = ['rx', 'tx']
+
+        # Force a first update because we need two update to have the first stat
+        self.update()
+        self.refresh_timer.set(0)
+
     def get_key(self):
         """Return the key of the list."""
         return 'interface_name'
 
-    @GlancesPlugin._check_decorator
+    # @GlancesPlugin._check_decorator
     @GlancesPlugin._log_result_decorator
     def update(self):
         """Update network stats using the input method.
@@ -124,6 +135,7 @@ class Plugin(GlancesPlugin):
                     tx = cumulative_tx - self.network_old[net].bytes_sent
                     cx = rx + tx
                     netstat = {'interface_name': n(net),
+                               'alias': self.has_alias(n(net)),
                                'time_since_update': time_since_update,
                                'cumulative_rx': cumulative_rx,
                                'rx': rx,
@@ -197,6 +209,7 @@ class Plugin(GlancesPlugin):
                         cx = rx + tx
                         netstat = {
                             'interface_name': interface_name,
+                            'alias': self.has_alias(interface_name),
                             'time_since_update': time_since_update,
                             'cumulative_rx': cumulative_rx,
                             'rx': rx,
@@ -223,13 +236,17 @@ class Plugin(GlancesPlugin):
         # Call the father's method
         super(Plugin, self).update_views()
 
+        # Check if the stats should be hidden
+        self.update_views_hidden()
+
         # Add specifics informations
         # Alert
-        for i in self.stats:
+        for i in self.get_raw():
             ifrealname = i['interface_name'].split(':')[0]
-            # Convert rate in bps ( to be able to compare to interface speed)
+            # Convert rate in bps (to be able to compare to interface speed)
             bps_rx = int(i['rx'] // i['time_since_update'] * 8)
             bps_tx = int(i['tx'] // i['time_since_update'] * 8)
+
             # Decorate the bitrate with the configuration file thresolds
             alert_rx = self.get_alert(bps_rx, header=ifrealname + '_rx')
             alert_tx = self.get_alert(bps_tx, header=ifrealname + '_tx')
@@ -290,12 +307,15 @@ class Plugin(GlancesPlugin):
             # Do not display interface in down state (issue #765)
             if ('is_up' in i) and (i['is_up'] is False):
                 continue
+            # Hide stats if never be different from 0 (issue #1787)
+            if all([self.get_views(item=i[self.get_key()], key=f, option='hidden') for f in self.hide_zero_fields]):
+                continue
             # Format stats
             # Is there an alias for the interface name ?
-            ifrealname = i['interface_name'].split(':')[0]
-            ifname = self.has_alias(i['interface_name'])
-            if ifname is None:
-                ifname = ifrealname
+            if i['alias'] is None:
+                ifname = i['interface_name'].split(':')[0]
+            else:
+                ifname = i['alias']
             if len(ifname) > name_max_width:
                 # Cut interface name if it is too long
                 ifname = '_' + ifname[-name_max_width + 1:]
