@@ -95,7 +95,15 @@ def get_smart_data():
     """
     stats = []
     # get all devices
-    devlist = DeviceList()
+    try:
+        devlist = DeviceList()
+    except TypeError as e:
+        # Catch error  (see #1806)
+        logger.debug(
+            'Smart plugin error - Can not grab device list ({})'.format(e))
+        global import_error_tag
+        import_error_tag = True
+        return stats
 
     for dev in devlist.devices:
         stats.append({
@@ -113,6 +121,7 @@ def get_smart_data():
                     assert num is not None
                 except Exception as e:
                     # we should never get here, but if we do, continue to next iteration and skip this attribute
+                    logger.debug('Smart plugin error - Skip the attribute {} ({})'.format(attribute, e))
                     continue
 
                 stats[-1][num] = attribdict
@@ -171,7 +180,7 @@ class Plugin(GlancesPlugin):
         ret = []
 
         # Only process if stats exist...
-        if not self.stats or self.is_disable():
+        if import_error_tag or not self.stats or self.is_disable():
             return ret
 
         # Max size for the interface name
@@ -191,7 +200,7 @@ class Plugin(GlancesPlugin):
             for smart_stat in sorted([i for i in device_stat.keys() if i != 'DeviceName'], key=int):
                 ret.append(self.curse_new_line())
                 msg = ' {:{width}}'.format(device_stat[smart_stat]['name'][:name_max_width-1].replace('_', ' '),
-                                          width=name_max_width-1)
+                                           width=name_max_width-1)
                 ret.append(self.curse_add_line(msg))
                 msg = '{:>8}'.format(device_stat[smart_stat]['raw'])
                 ret.append(self.curse_add_line(msg))
