@@ -2,7 +2,7 @@
 #
 # This file is part of Glances.
 #
-# Copyright (C) 2019 Nicolargo <nicolas@nicolargo.com>
+# Copyright (C) 2021 Nicolargo <nicolas@nicolargo.com>
 #
 # Glances is free software; you can redistribute it and/or modify
 # it under the terms of the GNU Lesser General Public License as published by
@@ -25,8 +25,9 @@ import sys
 import threading
 import traceback
 
-from glances.globals import exports_path, plugins_path, sys_path
 from glances.logger import logger
+from glances.globals import exports_path, plugins_path, sys_path
+from glances.timer import Counter
 
 
 class GlancesStats(object):
@@ -44,11 +45,9 @@ class GlancesStats(object):
         self.args = args
 
         # Load plugins and exports modules
+        self.first_export = True
         self.load_modules(self.args)
 
-        # Load the limits (for plugins)
-        self.load_limits(self.config)
-
     def __getattr__(self, item):
         """Overwrite the getattr method in case of attribute is not found.
 
@@ -109,14 +108,13 @@ class GlancesStats(object):
         # for example, the file glances_xxx.py
         # generate self._plugins_list["xxx"] = ...
         name = plugin_script[len(self.header):-3].lower()
+
+        # Loaf the plugin class
         try:
             # Import the plugin
             plugin = __import__(plugin_script[:-3])
             # Init and add the plugin to the dictionary
-            if name in ('help', 'amps', 'ports', 'folders'):
-                self._plugins[name] = plugin.Plugin(args=args, config=config)
-            else:
-                self._plugins[name] = plugin.Plugin(args=args)
+            self._plugins[name] = plugin.Plugin(args=args, config=config)
         except Exception as e:
             # If a plugin can not be loaded, display a critical message
             # on the console but do not crash
@@ -136,13 +134,17 @@ class GlancesStats(object):
 
     def load_plugins(self, args=None):
         """Load all plugins in the 'plugins' folder."""
+        start_duration = Counter()
         for item in os.listdir(plugins_path):
             if (item.startswith(self.header) and
                     item.endswith(".py") and
                     item != (self.header + "plugin.py")):
                 # Load the plugin
+                start_duration.reset()
                 self._load_plugin(os.path.basename(item),
                                   args=args, config=self.config)
+                logger.debug("Plugin {} started in {} seconds".format(item,
+                                                                      start_duration.get()))
 
         # Log plugins list
         logger.debug("Active plugins list: {}".format(self.getPluginsList()))
@@ -236,16 +238,21 @@ class GlancesStats(object):
 
         Each export module is ran in a dedicated thread.
         """
-        # threads = []
+        if self.first_export:
+            logger.debug("Do not export stats during the first iteration because some information are missing")
+            self.first_export = False
+            return False
+
         input_stats = input_stats or {}
 
         for e in self._exports:
             logger.debug("Export stats using the %s module" % e)
             thread = threading.Thread(target=self._exports[e].update,
                                       args=(input_stats,))
-            # threads.append(thread)
             thread.start()
 
+        return True
+
     def getAll(self):
         """Return all the stats (list)."""
         return [self._plugins[p].get_raw() for p in self._plugins]
